//
//  Counter.m
//  A simple program to count the number of lines, words, and chars in a file
//
//  Created by Rene Hexel on 25/10/06.
//  Copyright 2006, 2008 Rene Hexel. All rights reserved.
//

#import "Counter.h"
#include <stdio.h>

@implementation Counter

/**
 * Initialiser to count the words in a given text string
 */
- initWithString: (NSString *) text
{
	if (!(self = [self init]))		// invoke default initialiser
		return nil;

	content = text;				// use the text for counting
	
	return self;
}


/**
 * Initialiser to count the words in a given file
 */
- initWithFile: (NSString *) name		// count word in given file
{
	// read the file content into string "text"
	NSString *text = [NSString stringWithContentsOfFile: name];

	return [self initWithString: text];	// do the counting in the text
}


/**
 * do the actual counting of lines, words, and characters
 */
- count						// do the actual counting
{
	nChars = [content length];		// length in characters

	NSScanner *reader = [NSScanner scannerWithString: content];
	
	if (reader == nil) return nil;		// no content?
	
	// only skip white space characters (not new lines)
	[reader setCharactersToBeSkipped:
		[NSCharacterSet whitespaceCharacterSet]];

	while (![reader isAtEnd])
	{
		NSString *line;			// one line of text

		[reader scanUpToString: @"\n" intoString: &line];
		nLines++;
		
		NSScanner *lineScanner = [NSScanner scannerWithString: line];

		if (lineScanner == nil) return nil;

		while (![lineScanner isAtEnd])
		{
			if ([lineScanner scanUpToCharactersFromSet:
			     [NSCharacterSet whitespaceAndNewlineCharacterSet]
							intoString: NULL])
				nWords++;
		}

		// skip the new line character
		[reader scanString: @"\n" intoString: NULL];
	}
	
	return self;
}


/**
 * return the number of lines counted
 */
- (int) lines
{
	return nLines;
}


/**
 * return the number of words counted
 */
- (int) words
{
	return nWords;
}

/**
 * return the number of characters counted
 */
- (int) characters
{
	return nChars;
}
@end
